@@ -2,7 +2,7 @@
  *
  *
  *
- * Copyright (C) 1997-2013 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -20,6 +20,7 @@
 #include "util.h"
 #include "language.h"
 #include "lang_cfg.h"
+#include "vhdldocgen.h"
 #include "translator.h"
 #include "translator_en.h"
 #if !defined(ENGLISH_ONLY)
@@ -152,21 +153,11 @@
 #endif
 #endif // !ENGLISH_ONLY
 
-#define L_EQUAL(a) !qstricmp(langName,a)
+#define L_EQUAL(a) !qstricmp(langName.data(),a)
 
 Translator *theTranslator=0;
 
-static const char obsoleteMsg[] =
-        "---------\n"
-        "ERROR: The selected language is no longer supported!\n"
-        "If you want doxygen to produce output in this language \n"
-        "you are kindly requested to help bringing the documentation \n"
-        "up to date. Please read the development section of the manual \n"
-        "for more information or contact Petr Prikryl (Prikryl@skil.cz).\n"
-        "Thanks in advance!\n"
-        "---------\n";
-
-bool setTranslator(const char *langName)
+bool setTranslator(const QCString &langName)
 {
   if (L_EQUAL("english"))
   {
@@ -354,9 +345,9 @@ bool setTranslator(const char *langName)
   }
 #endif
 #ifdef LANG_SC
-  else if (L_EQUAL("serbian-cyrilic"))
+  else if (L_EQUAL("serbian-cyrillic") || L_EQUAL("serbiancyr")) /* serbiancyr for consistency with older versions */
   {
-    theTranslator=new TranslatorSerbian;
+    theTranslator=new TranslatorSerbianCyrillic;
   }
 #endif
 #ifdef LANG_CA
@@ -427,6 +418,6 @@ bool setTranslator(const char *langName)
   }
 
   QCString msg = theTranslator->updateNeededMessage();
-  if (!msg.isEmpty()) warn_uncond(msg);
+  if (!msg.isEmpty()) warn_uncond("%s", qPrint(msg));
   return TRUE;
 }
