@@ -1,12 +1,10 @@
 /******************************************************************************
  *
- * 
- *
- * Copyright (C) 1997-2014 by Dimitri van Heesch.
+ * Copyright (C) 1997-2020 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -18,48 +16,41 @@
 #ifndef OUTPUTLIST_H
 #define OUTPUTLIST_H
 
-#include <qlist.h>
+#include <utility>
+#include <vector>
+#include <memory>
+
 #include "index.h" // for IndexSections
 #include "outputgen.h"
 
-#define FORALLPROTO1(arg1) \
-  void forall(void (OutputGenerator::*func)(arg1),arg1)
-#define FORALLPROTO2(arg1,arg2) \
-  void forall(void (OutputGenerator::*func)(arg1,arg2),arg1,arg2)
-#define FORALLPROTO3(arg1,arg2,arg3) \
-  void forall(void (OutputGenerator::*func)(arg1,arg2,arg3),arg1,arg2,arg3)
-#define FORALLPROTO4(arg1,arg2,arg3,arg4) \
-  void forall(void (OutputGenerator::*func)(arg1,arg2,arg3,arg4),arg1,arg2,arg3,arg4)
-#define FORALLPROTO5(arg1,arg2,arg3,arg4,arg5) \
-  void forall(void (OutputGenerator::*func)(arg1,arg2,arg3,arg4,arg5),arg1,arg2,arg3,arg4,arg5)
-#define FORALLPROTO6(arg1,arg2,arg3,arg4,arg5,arg6) \
-  void forall(void (OutputGenerator::*func)(arg1,arg2,arg3,arg4,arg5,arg6),arg1,arg2,arg3,arg4,arg5,arg6)
-#define FORALLPROTO7(arg1,arg2,arg3,arg4,arg5,arg6,arg7) \
-  void forall(void (OutputGenerator::*func)(arg1,arg2,arg3,arg4,arg5,arg6,arg7),arg1,arg2,arg3,arg4,arg5,arg6,arg7)
-#define FORALLPROTO8(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8) \
-  void forall(void (OutputGenerator::*func)(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8),arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8)
-  
 class ClassDiagram;
 class DotClassGraph;
 class DotDirDeps;
 class DotInclDepGraph;
 class DotGfxHierarchyTable;
-class SectionDict;
 class DotGroupCollaboration;
 class DocRoot;
 
 /** Class representing a list of output generators that are written to
- *  in parallel. 
+ *  in parallel.
  */
 class OutputList : public OutputDocInterface
 {
   public:
-    OutputList(bool);
+    OutputList();
+    OutputList(const OutputList &ol);
+    OutputList &operator=(const OutputList &ol);
     virtual ~OutputList();
 
-    void add(const OutputGenerator *);
-    uint count() const { return m_outputs.count(); }
-    
+    template<class Generator>
+    void add()
+    {
+      m_outputs.emplace_back(std::make_unique<Generator>());
+    }
+
+    size_t size() const { return m_outputs.size(); }
+    int id() const { return m_id; }
+
     void disableAllBut(OutputGenerator::OutputType o);
     void enableAll();
     void disableAll();
@@ -74,57 +65,58 @@ class OutputList : public OutputDocInterface
     // OutputDocInterface implementation
     //////////////////////////////////////////////////
 
-    bool generateDoc(const char *fileName,int startLine,
-                     Definition *ctx,MemberDef *md,const QCString &docStr,
-                     bool indexWords,bool isExample,const char *exampleName=0,
-                     bool singleLine=FALSE,bool linkFromIndex=FALSE);
-    void writeDoc(DocRoot *root,Definition *ctx,MemberDef *md);
-    bool parseText(const QCString &textStr);
-    
+    void generateDoc(const QCString &fileName,int startLine,
+                     const Definition *ctx,const MemberDef *md,const QCString &docStr,
+                     bool indexWords,bool isExample,const QCString &exampleName /*=0*/,
+                     bool singleLine /*=FALSE*/,bool linkFromIndex /*=FALSE*/,
+                     bool markdownSupport /*=FALSE*/);
+    void writeDoc(DocRoot *root,const Definition *ctx,const MemberDef *md,int id=0);
+    void parseText(const QCString &textStr);
 
     void startIndexSection(IndexSections is)
     { forall(&OutputGenerator::startIndexSection,is); }
     void endIndexSection(IndexSections is)
     { forall(&OutputGenerator::endIndexSection,is); }
-    void writePageLink(const char *name,bool first)
+    void writePageLink(const QCString &name,bool first)
     { forall(&OutputGenerator::writePageLink,name,first); }
     void startProjectNumber()
     { forall(&OutputGenerator::startProjectNumber); }
     void endProjectNumber()
     { forall(&OutputGenerator::endProjectNumber); }
-    void writeStyleInfo(int part) 
+    void writeStyleInfo(int part)
     { forall(&OutputGenerator::writeStyleInfo,part); }
-    void startFile(const char *name,const char *manName,const char *title)
-    { forall(&OutputGenerator::startFile,name,manName,title); }
+    void startFile(const QCString &name,const QCString &manName,const QCString &title)
+    {
+      newId();
+      forall(&OutputGenerator::startFile,name,manName,title,m_id);
+    }
     void writeSearchInfo()
     { forall(&OutputGenerator::writeSearchInfo); }
-    void writeFooter(const char *navPath)
+    void writeFooter(const QCString &navPath)
     { forall(&OutputGenerator::writeFooter,navPath); }
-    void endFile() 
+    void endFile()
     { forall(&OutputGenerator::endFile); }
-    void startTitleHead(const char *fileName) 
+    void startTitleHead(const QCString &fileName)
     { forall(&OutputGenerator::startTitleHead,fileName); }
-    void endTitleHead(const char *fileName,const char *name)
+    void endTitleHead(const QCString &fileName,const QCString &name)
     { forall(&OutputGenerator::endTitleHead,fileName,name); }
-    void startTitle() 
+    void startTitle()
     { forall(&OutputGenerator::startTitle); }
-    void endTitle() 
+    void endTitle()
     { forall(&OutputGenerator::endTitle); }
-    //void newParagraph() 
-    //{ forall(&OutputGenerator::newParagraph); }
-    void startParagraph() 
-    { forall(&OutputGenerator::startParagraph); }
-    void endParagraph() 
+    void startParagraph(const QCString &classDef=QCString())
+    { forall(&OutputGenerator::startParagraph,classDef); }
+    void endParagraph()
     { forall(&OutputGenerator::endParagraph); }
-    void writeString(const char *text) 
+    void writeString(const QCString &text)
     { forall(&OutputGenerator::writeString,text); }
-    void startIndexListItem() 
+    void startIndexListItem()
     { forall(&OutputGenerator::startIndexListItem); }
-    void endIndexListItem() 
+    void endIndexListItem()
     { forall(&OutputGenerator::endIndexListItem); }
-    void startIndexList() 
+    void startIndexList()
     { forall(&OutputGenerator::startIndexList); }
-    void endIndexList() 
+    void endIndexList()
     { forall(&OutputGenerator::endIndexList); }
     void startIndexKey()
     { forall(&OutputGenerator::startIndexKey); }
@@ -132,56 +124,55 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::endIndexKey); }
     void startIndexValue(bool b)
     { forall(&OutputGenerator::startIndexValue,b); }
-    void endIndexValue(const char *name,bool b)
+    void endIndexValue(const QCString &name,bool b)
     { forall(&OutputGenerator::endIndexValue,name,b); }
-    void startItemList() 
+    void startItemList()
     { forall(&OutputGenerator::startItemList); }
-    void endItemList() 
+    void endItemList()
     { forall(&OutputGenerator::endItemList); }
-    void startIndexItem(const char *ref,const char *file)
+    void startIndexItem(const QCString &ref,const QCString &file)
     { forall(&OutputGenerator::startIndexItem,ref,file); }
-    void endIndexItem(const char *ref,const char *file)
+    void endIndexItem(const QCString &ref,const QCString &file)
     { forall(&OutputGenerator::endIndexItem,ref,file); }
-    void docify(const char *s)
+    void docify(const QCString &s)
     { forall(&OutputGenerator::docify,s); }
-    void codify(const char *s)
+    void codify(const QCString &s)
     { forall(&OutputGenerator::codify,s); }
-    void writeObjectLink(const char *ref,const char *file,
-                         const char *anchor, const char *name)
+    void writeObjectLink(const QCString &ref,const QCString &file,
+                         const QCString &anchor, const QCString &name)
     { forall(&OutputGenerator::writeObjectLink,ref,file,anchor,name); }
-    void writeCodeLink(const char *ref,const char *file,
-                       const char *anchor,const char *name,
-                       const char *tooltip)
-    { forall(&OutputGenerator::writeCodeLink,ref,file,anchor,name,tooltip); }
-    void writeTooltip(const char *id, const DocLinkInfo &docInfo, const char *decl,
-                      const char *desc, const SourceLinkInfo &defInfo, const SourceLinkInfo &declInfo)
+    void writeCodeLink(CodeSymbolType type,
+                       const QCString &ref,const QCString &file,
+                       const QCString &anchor,const QCString &name,
+                       const QCString &tooltip)
+    { forall(&OutputGenerator::writeCodeLink,type,ref,file,anchor,name,tooltip); }
+    void writeTooltip(const QCString &id, const DocLinkInfo &docInfo, const QCString &decl,
+                      const QCString &desc, const SourceLinkInfo &defInfo, const SourceLinkInfo &declInfo)
     { forall(&OutputGenerator::writeTooltip,id,docInfo,decl,desc,defInfo,declInfo); }
-    void startTextLink(const char *file,const char *anchor)
+    void startTextLink(const QCString &file,const QCString &anchor)
     { forall(&OutputGenerator::startTextLink,file,anchor); }
     void endTextLink()
     { forall(&OutputGenerator::endTextLink); }
-    void startHtmlLink(const char *url)
+    void startHtmlLink(const QCString &url)
     { forall(&OutputGenerator::startHtmlLink,url); }
     void endHtmlLink()
     { forall(&OutputGenerator::endHtmlLink); }
-    void writeStartAnnoItem(const char *type,const char *file, 
-                            const char *path,const char *name)
+    void writeStartAnnoItem(const QCString &type,const QCString &file,
+                            const QCString &path,const QCString &name)
     { forall(&OutputGenerator::writeStartAnnoItem,type,file,path,name); }
-    void writeEndAnnoItem(const char *name)
+    void writeEndAnnoItem(const QCString &name)
     { forall(&OutputGenerator::writeEndAnnoItem,name); }
-    void startTypewriter() 
+    void startTypewriter()
     { forall(&OutputGenerator::startTypewriter); }
-    void endTypewriter() 
+    void endTypewriter()
     { forall(&OutputGenerator::endTypewriter); }
     void startGroupHeader(int extraLevels=0)
     { forall(&OutputGenerator::startGroupHeader,extraLevels); }
     void endGroupHeader(int extraLevels=0)
     { forall(&OutputGenerator::endGroupHeader,extraLevels); }
-    //void writeListItem() 
-    //{ forall(&OutputGenerator::writeListItem); }
-    void startItemListItem() 
+    void startItemListItem()
     { forall(&OutputGenerator::startItemListItem); }
-    void endItemListItem() 
+    void endItemListItem()
     { forall(&OutputGenerator::endItemListItem); }
     void startMemberSections()
     { forall(&OutputGenerator::startMemberSections); }
@@ -191,39 +182,43 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startHeaderSection); }
     void endHeaderSection()
     { forall(&OutputGenerator::endHeaderSection); }
-    void startMemberHeader(const char *anchor)
-    { forall(&OutputGenerator::startMemberHeader,anchor); }
+    void startMemberHeader(const QCString &anchor, int typ = 2)
+    { forall(&OutputGenerator::startMemberHeader,anchor,typ); }
     void endMemberHeader()
     { forall(&OutputGenerator::endMemberHeader); }
     void startMemberSubtitle()
     { forall(&OutputGenerator::startMemberSubtitle); }
     void endMemberSubtitle()
     { forall(&OutputGenerator::endMemberSubtitle); }
-    void startMemberDocList() 
+    void startMemberDocList()
     { forall(&OutputGenerator::startMemberDocList); }
-    void endMemberDocList() 
+    void endMemberDocList()
     { forall(&OutputGenerator::endMemberDocList); }
-    void startMemberList() 
+    void startMemberList()
     { forall(&OutputGenerator::startMemberList); }
-    void endMemberList() 
+    void endMemberList()
     { forall(&OutputGenerator::endMemberList); }
     void startInlineHeader()
     { forall(&OutputGenerator::startInlineHeader); }
     void endInlineHeader()
     { forall(&OutputGenerator::endInlineHeader); }
-    void startAnonTypeScope(int i1) 
+    void startAnonTypeScope(int i1)
     { forall(&OutputGenerator::startAnonTypeScope,i1); }
-    void endAnonTypeScope(int i1) 
+    void endAnonTypeScope(int i1)
     { forall(&OutputGenerator::endAnonTypeScope,i1); }
-    void startMemberItem(const char *anchor,int i1,const char *id=0) 
+    void startMemberItem(const QCString &anchor,int i1,const QCString &id=QCString())
     { forall(&OutputGenerator::startMemberItem,anchor,i1,id); }
-    void endMemberItem() 
+    void endMemberItem()
     { forall(&OutputGenerator::endMemberItem); }
-    void startMemberTemplateParams() 
+    void startMemberTemplateParams()
     { forall(&OutputGenerator::startMemberTemplateParams); }
-    void endMemberTemplateParams(const char *anchor,const char *inheritId) 
+    void endMemberTemplateParams(const QCString &anchor,const QCString &inheritId)
     { forall(&OutputGenerator::endMemberTemplateParams,anchor,inheritId); }
-    void startMemberGroupHeader(bool b) 
+    void startCompoundTemplateParams()
+    { forall(&OutputGenerator::startCompoundTemplateParams); }
+    void endCompoundTemplateParams()
+    { forall(&OutputGenerator::endCompoundTemplateParams); }
+    void startMemberGroupHeader(bool b)
     { forall(&OutputGenerator::startMemberGroupHeader,b); }
     void endMemberGroupHeader()
     { forall(&OutputGenerator::endMemberGroupHeader); }
@@ -235,142 +230,152 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startMemberGroup); }
     void endMemberGroup(bool last)
     { forall(&OutputGenerator::endMemberGroup,last); }
-    void insertMemberAlign(bool templ=FALSE) 
+    void insertMemberAlign(bool templ=FALSE)
     { forall(&OutputGenerator::insertMemberAlign,templ); }
-    void writeRuler() 
+    void insertMemberAlignLeft(int typ=0, bool templ=FALSE)
+    { forall(&OutputGenerator::insertMemberAlignLeft,typ,templ); }
+    void writeRuler()
     { forall(&OutputGenerator::writeRuler); }
-    void writeAnchor(const char *fileName,const char *name)
+    void writeAnchor(const QCString &fileName,const QCString &name)
     { forall(&OutputGenerator::writeAnchor,fileName,name); }
-    void startCodeFragment() 
-    { forall(&OutputGenerator::startCodeFragment); }
-    void endCodeFragment() 
-    { forall(&OutputGenerator::endCodeFragment); }
-    void startCodeLine(bool hasLineNumbers) 
+    void startCodeFragment(const QCString &style)
+    { forall(&OutputGenerator::startCodeFragment,style); }
+    void endCodeFragment(const QCString &style)
+    { forall(&OutputGenerator::endCodeFragment,style); }
+    void startCodeLine(bool hasLineNumbers)
     { forall(&OutputGenerator::startCodeLine,hasLineNumbers); }
-    void endCodeLine() 
+    void endCodeLine()
     { forall(&OutputGenerator::endCodeLine); }
-    void writeLineNumber(const char *ref,const char *file,const char *anchor,
-                         int lineNumber) 
+    void writeLineNumber(const QCString &ref,const QCString &file,const QCString &anchor,
+                         int lineNumber)
     { forall(&OutputGenerator::writeLineNumber,ref,file,anchor,lineNumber); }
-    void startEmphasis() 
+    void startEmphasis()
     { forall(&OutputGenerator::startEmphasis); }
-    void endEmphasis() 
+    void endEmphasis()
     { forall(&OutputGenerator::endEmphasis); }
     void writeChar(char c)
     { forall(&OutputGenerator::writeChar,c); }
-    void startMemberDoc(const char *clName,const char *memName,
-                        const char *anchor,const char *title,bool showInline)
-    { forall(&OutputGenerator::startMemberDoc,clName,memName,anchor,title,showInline); }
-    void endMemberDoc(bool hasArgs) 
+    void startMemberDoc(const QCString &clName,const QCString &memName,
+                        const QCString &anchor,const QCString &title,
+                        int memCount,int memTotal,bool showInline)
+    { forall(&OutputGenerator::startMemberDoc,clName,memName,anchor,title,memCount,memTotal,showInline); }
+    void endMemberDoc(bool hasArgs)
     { forall(&OutputGenerator::endMemberDoc,hasArgs); }
-    void startDoxyAnchor(const char *fName,const char *manName,
-                         const char *anchor, const char *name,
-                         const char *args)
+    void startDoxyAnchor(const QCString &fName,const QCString &manName,
+                         const QCString &anchor, const QCString &name,
+                         const QCString &args)
     { forall(&OutputGenerator::startDoxyAnchor,fName,manName,anchor,name,args); }
-    void endDoxyAnchor(const char *fn,const char *anchor)
+    void endDoxyAnchor(const QCString &fn,const QCString &anchor)
     { forall(&OutputGenerator::endDoxyAnchor,fn,anchor); }
-    void writeLatexSpacing() 
+    void writeLatexSpacing()
     { forall(&OutputGenerator::writeLatexSpacing); }
-    void startDescription() 
+    void startDescription()
     { forall(&OutputGenerator::startDescription); }
-    void endDescription() 
+    void endDescription()
     { forall(&OutputGenerator::endDescription); }
-    void startDescItem() 
+    void startDescItem()
     { forall(&OutputGenerator::startDescItem); }
-    void endDescItem() 
+    void endDescItem()
     { forall(&OutputGenerator::endDescItem); }
-    void startDescForItem() 
+    void startDescForItem()
     { forall(&OutputGenerator::startDescForItem); }
-    void endDescForItem() 
+    void endDescForItem()
     { forall(&OutputGenerator::endDescForItem); }
-    void startSubsection() 
+    void startSubsection()
     { forall(&OutputGenerator::startSubsection); }
-    void endSubsection() 
+    void endSubsection()
     { forall(&OutputGenerator::endSubsection); }
-    void startSubsubsection() 
+    void startSubsubsection()
     { forall(&OutputGenerator::startSubsubsection); }
-    void endSubsubsection() 
+    void endSubsubsection()
     { forall(&OutputGenerator::endSubsubsection); }
-    void startCenter() 
+    void startCenter()
     { forall(&OutputGenerator::startCenter); }
-    void endCenter() 
+    void endCenter()
     { forall(&OutputGenerator::endCenter); }
-    void startSmall() 
+    void startSmall()
     { forall(&OutputGenerator::startSmall); }
-    void endSmall() 
+    void endSmall()
     { forall(&OutputGenerator::endSmall); }
-    void lineBreak(const char *style=0) 
+    void lineBreak(const QCString &style=QCString())
     { forall(&OutputGenerator::lineBreak,style); }
-    void startBold() 
+    void startBold()
     { forall(&OutputGenerator::startBold); }
-    void endBold() 
+    void endBold()
     { forall(&OutputGenerator::endBold); }
-    void startMemberDescription(const char *anchor,const char *inheritId=0) 
-    { forall(&OutputGenerator::startMemberDescription,anchor,inheritId); }
-    void endMemberDescription() 
+    void startMemberDescription(const QCString &anchor,const QCString &inheritId=QCString(), bool typ = false)
+    { forall(&OutputGenerator::startMemberDescription,anchor,inheritId, typ); }
+    void endMemberDescription()
     { forall(&OutputGenerator::endMemberDescription); }
     void startMemberDeclaration()
     { forall(&OutputGenerator::startMemberDeclaration); }
-    void endMemberDeclaration(const char *anchor,const char *inheritId)
+    void endMemberDeclaration(const QCString &anchor,const QCString &inheritId)
     { forall(&OutputGenerator::endMemberDeclaration,anchor,inheritId); }
-    void writeInheritedSectionTitle(const char *id,   const char *ref,
-                                    const char *file, const char *anchor,
-                                    const char *title,const char *name)
+    void writeInheritedSectionTitle(const QCString &id,   const QCString &ref,
+                                    const QCString &file, const QCString &anchor,
+                                    const QCString &title,const QCString &name)
     { forall(&OutputGenerator::writeInheritedSectionTitle,id,ref,
                                     file,anchor,title,name); }
-    void startSimpleSect(SectionTypes t,const char *file,const char *anchor,
-                         const char *title) 
-    { forall(&OutputGenerator::startSimpleSect,t,file,anchor,title); }
-    void endSimpleSect() 
-    { forall(&OutputGenerator::endSimpleSect); }
-    void startParamList(ParamListTypes t,const char *title) 
+    void startExamples()
+    { forall(&OutputGenerator::startExamples); }
+    void endExamples()
+    { forall(&OutputGenerator::endExamples); }
+    void startParamList(ParamListTypes t,const QCString &title)
     { forall(&OutputGenerator::startParamList,t,title); }
-    void endParamList() 
+    void endParamList()
     { forall(&OutputGenerator::endParamList); }
-    void startIndent() 
+    void startIndent()
     { forall(&OutputGenerator::startIndent); }
-    void endIndent() 
+    void endIndent()
     { forall(&OutputGenerator::endIndent); }
-    void startSection(const char *lab,const char *title,SectionInfo::SectionType t)
+    void startSection(const QCString &lab,const QCString &title,SectionType t)
     { forall(&OutputGenerator::startSection,lab,title,t); }
-    void endSection(const char *lab,SectionInfo::SectionType t)
+    void endSection(const QCString &lab,SectionType t)
     { forall(&OutputGenerator::endSection,lab,t); }
-    void addIndexItem(const char *s1,const char *s2)
+    void addIndexItem(const QCString &s1,const QCString &s2)
     { forall(&OutputGenerator::addIndexItem,s1,s2); }
-    void writeSynopsis() 
+    void writeSynopsis()
     { forall(&OutputGenerator::writeSynopsis); }
     void startClassDiagram()
     { forall(&OutputGenerator::startClassDiagram); }
-    void endClassDiagram(const ClassDiagram &d,const char *f,const char *n)
+    void endClassDiagram(const ClassDiagram &d,const QCString &f,const QCString &n)
     { forall(&OutputGenerator::endClassDiagram,d,f,n); }
     void startPageRef()
     { forall(&OutputGenerator::startPageRef); }
-    void endPageRef(const char *c,const char *a)
+    void endPageRef(const QCString &c,const QCString &a)
     { forall(&OutputGenerator::endPageRef,c,a); }
     void startQuickIndices()
     { forall(&OutputGenerator::startQuickIndices); }
     void endQuickIndices()
     { forall(&OutputGenerator::endQuickIndices); }
-    void writeSplitBar(const char *name)
+    void writeSplitBar(const QCString &name)
     { forall(&OutputGenerator::writeSplitBar,name); }
-    void writeNavigationPath(const char *s)
+    void writeNavigationPath(const QCString &s)
     { forall(&OutputGenerator::writeNavigationPath,s); }
     void writeLogo()
     { forall(&OutputGenerator::writeLogo); }
-    void writeQuickLinks(bool compact,HighlightedItem hli,const char *file)
+    void writeQuickLinks(bool compact,HighlightedItem hli,const QCString &file)
     { forall(&OutputGenerator::writeQuickLinks,compact,hli,file); }
-    void writeSummaryLink(const char *file,const char *anchor,const char *title,bool first)
+    void writeSummaryLink(const QCString &file,const QCString &anchor,const QCString &title,bool first)
     { forall(&OutputGenerator::writeSummaryLink,file,anchor,title,first); }
     void startContents()
     { forall(&OutputGenerator::startContents); }
     void endContents()
     { forall(&OutputGenerator::endContents); }
+    void startPageDoc(const QCString &pageTitle)
+    { forall(&OutputGenerator::startPageDoc, pageTitle); }
+    void endPageDoc()
+    { forall(&OutputGenerator::endPageDoc); }
     void writeNonBreakableSpace(int num)
     { forall(&OutputGenerator::writeNonBreakableSpace,num); }
-    void startDescTable(const char *title)
+    void startDescTable(const QCString &title)
     { forall(&OutputGenerator::startDescTable,title); }
     void endDescTable()
     { forall(&OutputGenerator::endDescTable); }
+    void startDescTableRow()
+    { forall(&OutputGenerator::startDescTableRow); }
+    void endDescTableRow()
+    { forall(&OutputGenerator::endDescTableRow); }
     void startDescTableTitle()
     { forall(&OutputGenerator::startDescTableTitle); }
     void endDescTableTitle()
@@ -381,25 +386,25 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::endDescTableData); }
     void startDotGraph()
     { forall(&OutputGenerator::startDotGraph); }
-    void endDotGraph(const DotClassGraph &g)
+    void endDotGraph(DotClassGraph &g)
     { forall(&OutputGenerator::endDotGraph,g); }
     void startInclDepGraph()
     { forall(&OutputGenerator::startInclDepGraph); }
-    void endInclDepGraph(const DotInclDepGraph &g)
+    void endInclDepGraph(DotInclDepGraph &g)
     { forall(&OutputGenerator::endInclDepGraph,g); }
     void startCallGraph()
     { forall(&OutputGenerator::startCallGraph); }
-    void endCallGraph(const DotCallGraph &g)
+    void endCallGraph(DotCallGraph &g)
     { forall(&OutputGenerator::endCallGraph,g); }
     void startDirDepGraph()
     { forall(&OutputGenerator::startDirDepGraph); }
-    void endDirDepGraph(const DotDirDeps &g)
+    void endDirDepGraph(DotDirDeps &g)
     { forall(&OutputGenerator::endDirDepGraph,g); }
     void startGroupCollaboration()
     { forall(&OutputGenerator::startGroupCollaboration); }
-    void endGroupCollaboration(const DotGroupCollaboration &g)
+    void endGroupCollaboration(DotGroupCollaboration &g)
     { forall(&OutputGenerator::endGroupCollaboration,g); }
-    void writeGraphicalHierarchy(const DotGfxHierarchyTable &g)
+    void writeGraphicalHierarchy(DotGfxHierarchyTable &g)
     { forall(&OutputGenerator::writeGraphicalHierarchy,g); }
     void startTextBlock(bool dense=FALSE)
     { forall(&OutputGenerator::startTextBlock,dense); }
@@ -415,7 +420,7 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startMemberDocName,align); }
     void endMemberDocName()
     { forall(&OutputGenerator::endMemberDocName); }
-    void startParameterType(bool first,const char *key)
+    void startParameterType(bool first,const QCString &key)
     { forall(&OutputGenerator::startParameterType,first,key); }
     void endParameterType()
     { forall(&OutputGenerator::endParameterType); }
@@ -427,14 +432,14 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startParameterList,openBracket); }
     void endParameterList()
     { forall(&OutputGenerator::endParameterList); }
-    void exceptionEntry(const char* prefix,bool closeBracket)
+    void exceptionEntry(const QCString &prefix,bool closeBracket)
     { forall(&OutputGenerator::exceptionEntry,prefix,closeBracket); }
 
-    void startConstraintList(const char *header) 
+    void startConstraintList(const QCString &header)
     { forall(&OutputGenerator::startConstraintList,header); }
-    void startConstraintParam() 
+    void startConstraintParam()
     { forall(&OutputGenerator::startConstraintParam); }
-    void endConstraintParam() 
+    void endConstraintParam()
     { forall(&OutputGenerator::endConstraintParam); }
     void startConstraintType()
     { forall(&OutputGenerator::startConstraintType); }
@@ -447,10 +452,10 @@ class OutputList : public OutputDocInterface
     void endConstraintList()
     { forall(&OutputGenerator::endConstraintList); }
 
-    void startMemberDocSimple()
-    { forall(&OutputGenerator::startMemberDocSimple); }
-    void endMemberDocSimple()
-    { forall(&OutputGenerator::endMemberDocSimple); }
+    void startMemberDocSimple(bool b)
+    { forall(&OutputGenerator::startMemberDocSimple,b); }
+    void endMemberDocSimple(bool b)
+    { forall(&OutputGenerator::endMemberDocSimple,b); }
     void startInlineMemberType()
     { forall(&OutputGenerator::startInlineMemberType); }
     void endInlineMemberType()
@@ -464,91 +469,53 @@ class OutputList : public OutputDocInterface
     void endInlineMemberDoc()
     { forall(&OutputGenerator::endInlineMemberDoc); }
 
-    void startLabels() 
+    void startLabels()
     { forall(&OutputGenerator::startLabels); }
-    void writeLabel(const char *l,bool isLast)
+    void writeLabel(const QCString &l,bool isLast)
     { forall(&OutputGenerator::writeLabel,l,isLast); }
     void endLabels()
     { forall(&OutputGenerator::endLabels); }
 
-    void startFontClass(const char *c)
+    void cleanup()
+    { forall(&OutputGenerator::cleanup); }
+
+    void startFontClass(const QCString &c)
     { forall(&OutputGenerator::startFontClass,c); }
     void endFontClass()
     { forall(&OutputGenerator::endFontClass); }
-    void writeCodeAnchor(const char *name)
+    void writeCodeAnchor(const QCString &name)
     { forall(&OutputGenerator::writeCodeAnchor,name); }
-    void setCurrentDoc(Definition *context,const char *anchor,bool isSourceFile)
+    void setCurrentDoc(const Definition *context,const QCString &anchor,bool isSourceFile)
     { forall(&OutputGenerator::setCurrentDoc,context,anchor,isSourceFile); }
-    void addWord(const char *word,bool hiPriority)
+    void addWord(const QCString &word,bool hiPriority)
     { forall(&OutputGenerator::addWord,word,hiPriority); }
 
-    void startPlainFile(const char *name)
-    {
-      QListIterator<OutputGenerator> it(m_outputs);
-      OutputGenerator *og;
-      for (;(og=it.current());++it)
-      {
-        if (og->isEnabled()) (og->startPlainFile)(name);
-      }
-    }
+    void startPlainFile(const QCString &name)
+    { forall(&OutputGenerator::startPlainFile,name); }
     void endPlainFile()
-    {
-      QListIterator<OutputGenerator> it(m_outputs);
-      OutputGenerator *og;
-      for (;(og=it.current());++it)
-      {
-        if (og->isEnabled()) (og->endPlainFile)();
-      }
-    }
+    { forall(&OutputGenerator::endPlainFile); }
 
   private:
     void debug();
     void clear();
+    void newId();
 
-    void forall(void (OutputGenerator::*func)());
-    FORALLPROTO1(const char *);
-    FORALLPROTO1(char);
-    FORALLPROTO1(IndexSections);
-    FORALLPROTO1(int);
-    FORALLPROTO1(const DotClassGraph &);
-    FORALLPROTO1(const DotInclDepGraph &);
-    FORALLPROTO1(const DotCallGraph &);
-    FORALLPROTO1(const DotGroupCollaboration &);
-    FORALLPROTO1(const DotDirDeps &);
-    FORALLPROTO1(const DotGfxHierarchyTable &);
-    FORALLPROTO1(SectionTypes);
-#if defined(HAS_BOOL_TYPE) || defined(Q_HAS_BOOL_TYPE)
-    FORALLPROTO1(bool);
-    FORALLPROTO2(bool,int);
-    FORALLPROTO2(bool,bool);
-    FORALLPROTO2(const char *,bool);
-    FORALLPROTO4(const char *,const char *,const char *,int);
-#endif
-    FORALLPROTO2(int,bool);
-    FORALLPROTO2(bool,const char *);
-    FORALLPROTO2(ParamListTypes,const char *);
-    FORALLPROTO2(const char *,const char *);
-    FORALLPROTO2(const char *,int);
-    FORALLPROTO2(const char *,SectionInfo::SectionType);
-    FORALLPROTO3(bool,HighlightedItem,const char *);
-    FORALLPROTO3(bool,bool,bool);
-    FORALLPROTO3(const char *,const char *,bool);
-    FORALLPROTO3(const char *,int,const char *);
-    FORALLPROTO3(const char *,const char *,SectionInfo::SectionType);
-    FORALLPROTO3(uchar,uchar,uchar);
-    FORALLPROTO3(const char *,const char *,const char *);
-    FORALLPROTO3(const ClassDiagram &,const char *,const char *);
-    FORALLPROTO3(Definition*,const char *,bool);
-    FORALLPROTO4(SectionTypes,const char *,const char *,const char *);
-    FORALLPROTO4(const char *,const char *,const char *,const char *);
-    FORALLPROTO4(const char *,const char *,const char *,bool);
-    FORALLPROTO5(const char *,const char *,const char *,const char *,const char *);
-    FORALLPROTO5(const char *,const char *,const char *,const char *,bool);
-    FORALLPROTO6(const char *,const char *,const char *,const char *,const char *,const char *);
-    FORALLPROTO6(const char *,const DocLinkInfo &,const char *,const char *,const SourceLinkInfo &,const SourceLinkInfo &);
+    // For each output format that is enabled (OutputGenerator::isEnabled()) we forward
+    // the method call.
+    // We use C++11 variadic templates and perfect forwarding to implement forall() generically,
+    // and split the types of the methods from the arguments passed to allow implicit conversions.
+    template<typename T,class... Ts,class... As>
+    void forall(void (T::*methodPtr)(Ts...),As&&... args)
+    {
+      for (const auto &og : m_outputs)
+      {
+        if (og->isEnabled()) (og.get()->*methodPtr)(std::forward<As>(args)...);
+      }
+    }
+
+    std::vector< std::unique_ptr<OutputGenerator> > m_outputs;
+    int m_id;
 
-    OutputList(const OutputList &ol);
-    QList<OutputGenerator> m_outputs;
 };
 
 #endif
