@@ -1,13 +1,13 @@
 /******************************************************************************
  *
- * 
  *
  *
- * Copyright (C) 1997-2014 by Dimitri van Heesch.
+ *
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -16,30 +16,32 @@
  *
  */
 
-#ifndef _RTFDOCVISITOR_H
-#define _RTFDOCVISITOR_H
+#ifndef RTFDOCVISITOR_H
+#define RTFDOCVISITOR_H
+
+#include <iostream>
 
 #include "docvisitor.h"
-#include <qstack.h>
-#include <qcstring.h>
+#include "qcstring.h"
 
-class FTextStream;
 class CodeOutputInterface;
+class TextStream;
 
 /*! @brief Concrete visitor implementation for RTF output. */
 class RTFDocVisitor : public DocVisitor
 {
   public:
-    RTFDocVisitor(FTextStream &t,CodeOutputInterface &ci,const char *langExt);
-    
+    RTFDocVisitor(TextStream &t,CodeOutputInterface &ci,const QCString &langExt);
+
     //--------------------------------------
     // visitor functions for leaf nodes
     //--------------------------------------
-    
+
     void visit(DocWord *);
     void visit(DocLinkedWord *);
     void visit(DocWhiteSpace *);
     void visit(DocSymbol *);
+    void visit(DocEmoji *);
     void visit(DocURL *);
     void visit(DocLineBreak *);
     void visit(DocHorRuler *);
@@ -56,7 +58,7 @@ class RTFDocVisitor : public DocVisitor
     //--------------------------------------
     // visitor functions for compound nodes
     //--------------------------------------
-    
+
     void visitPre(DocAutoList *);
     void visitPost(DocAutoList *);
     void visitPre(DocAutoListItem *);
@@ -125,8 +127,6 @@ class RTFDocVisitor : public DocVisitor
     void visitPost(DocXRefItem *);
     void visitPre(DocInternalRef *);
     void visitPost(DocInternalRef *);
-    void visitPre(DocCopy *);
-    void visitPost(DocCopy *);
     void visitPre(DocText *);
     void visitPost(DocText *);
     void visitPre(DocHtmlBlockQuote *);
@@ -139,34 +139,35 @@ class RTFDocVisitor : public DocVisitor
   private:
 
     //--------------------------------------
-    // helper functions 
+    // helper functions
     //--------------------------------------
-    
-    void filter(const char *str,bool verbatim=FALSE);
+
+    void filter(const QCString &str,bool verbatim=FALSE);
     void startLink(const QCString &ref,const QCString &file,
                    const QCString &anchor);
     void endLink(const QCString &ref);
-    QCString getStyle(const char *name);
+    QCString getStyle(const QCString &name);
     void incIndentLevel();
     void decIndentLevel();
 
-    void pushEnabled();
-    void popEnabled();
-    void writeDotFile(const QCString &fileName);
-    void writeMscFile(const QCString &fileName);
-    void writeDiaFile(const QCString &fileName);
-    void writePlantUMLFile(const QCString &fileName);
+    void includePicturePreRTF(const QCString &name, bool isTypeRTF, bool hasCaption, bool inlineImage = FALSE);
+    void includePicturePostRTF(bool isTypeRTF, bool hasCaption, bool inlineImage = FALSE);
+    void writeDotFile(const QCString &fileName, bool hasCaption,const QCString &srcFile,int srcLine);
+    void writeDotFile(DocDotFile *);
+    void writeMscFile(const QCString &fileName, bool hasCaption,const QCString &srcFile,int srcLine);
+    void writeMscFile(DocMscFile *);
+    void writeDiaFile(DocDiaFile *);
+    void writePlantUMLFile(const QCString &fileName, bool hasCaption);
 
     //--------------------------------------
     // state variables
     //--------------------------------------
 
-    FTextStream &m_t;
+    TextStream &m_t;
     CodeOutputInterface &m_ci;
     bool m_insidePre;
     bool m_hide;
     int m_indentLevel;
-    QStack<bool> m_enabled;
     bool m_lastIsPara;
     QCString m_langExt;
 };
