@@ -85,7 +85,7 @@
 //    something else.  It is difficult to find the general translation
 //    for all kinds in the Czech language.
 
-class TranslatorCzech : public Translator
+class TranslatorCzech : public TranslatorAdapter_1_8_15
 {
   public:
     // --- Language control methods -------------------
@@ -96,7 +96,18 @@ class TranslatorCzech : public Translator
     virtual QCString latexLanguageSupportCommand()
     {
       return "\\usepackage[T2A]{fontenc}\n"
-             "\\usepackage[czech]{babel}\n";
+             "\\usepackage[czech]{babel}\n"
+             "\\usepackage{regexpatch}\n"
+             "\\makeatletter\n"
+             "% Change the `-` delimiter to an active character\n"
+             "\\xpatchparametertext\\@@@cmidrule{-}{\\cA-}{}{}\n"
+             "\\xpatchparametertext\\@cline{-}{\\cA-}{}{}\n"
+             "\\makeatother\n";
+    }
+
+    virtual QCString trISOLang()
+    {
+      return "cs";
     }
 
     // --- Language translation methods -------------------
@@ -128,7 +139,7 @@ class TranslatorCzech : public Translator
     /*! header that is put before the list of member attributes. */
     virtual QCString trMemberDataDocumentation()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Dokumentace k položkám";
       }
@@ -161,10 +172,10 @@ class TranslatorCzech : public Translator
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
      */
-    virtual QCString trGeneratedAutomatically(const char *s)
+    virtual QCString trGeneratedAutomatically(const QCString &s)
     { QCString result("Generováno automaticky programem Doxygen "
                       "ze zdrojových textů");
-      if (s) result += QCString(" projektu ") + s;
+      if (!s.isEmpty()) result += QCString(" projektu ") + s;
       result += ".";
       return result;
     }
@@ -196,7 +207,7 @@ class TranslatorCzech : public Translator
     /*! This is put above each page as a link to the list of annotated classes */
     virtual QCString trCompoundList()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Datové struktury";
       }
@@ -213,7 +224,7 @@ class TranslatorCzech : public Translator
     /*! This is put above each page as a link to all members of compounds. */
     virtual QCString trCompoundMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Datové položky";
       }
@@ -226,7 +237,7 @@ class TranslatorCzech : public Translator
     /*! This is put above each page as a link to all members of files. */
     virtual QCString trFileMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Globální symboly";
       }
@@ -266,7 +277,7 @@ class TranslatorCzech : public Translator
     /*! This is an introduction to the annotated compound list. */
     virtual QCString trCompoundListDescription()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Následující seznam obsahuje identifikace datových "
                       "struktur a jejich stručné popisy:";
@@ -290,7 +301,7 @@ class TranslatorCzech : public Translator
         result += "dokumentovaných ";
       }
 
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result += "položek struktur (struct) a unií (union) ";
       }
@@ -303,7 +314,7 @@ class TranslatorCzech : public Translator
 
       if (extractAll)
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result += "dokumentaci struktur/unií, ke kterým příslušejí:";
         }
@@ -314,7 +325,7 @@ class TranslatorCzech : public Translator
       }
       else
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result+="struktury/unie, ke kterým příslušejí:";
         }
@@ -333,7 +344,7 @@ class TranslatorCzech : public Translator
       QCString result="Zde naleznete seznam všech ";
       if (!extractAll) result+="dokumentovaných ";
 
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result+="funkcí, proměnných, maker, výčtů a definic typů (typedef) "
                 "s odkazy na ";
@@ -388,7 +399,7 @@ class TranslatorCzech : public Translator
      */
     virtual QCString trCompoundIndex()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Rejstřík datových struktur";
       }
@@ -415,7 +426,7 @@ class TranslatorCzech : public Translator
      */
     virtual QCString trClassDocumentation()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Dokumentace datových struktur";
       }
@@ -518,7 +529,7 @@ class TranslatorCzech : public Translator
      */
     virtual QCString trCompounds()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Datové struktry";
       }
@@ -531,18 +542,17 @@ class TranslatorCzech : public Translator
     /*! This is used in the standard footer of each page and indicates when
      *  the page was generated
      */
-    virtual QCString trGeneratedAt(const char *date,const char *projName)
+    virtual QCString trGeneratedAt(const QCString &date,const QCString &projName)
     {
       QCString result("Generováno ");
       result += date;
-      if (projName)
-          result += QCString(" pro projekt ") + projName;
+      if (!projName.isEmpty()) result += QCString(" pro projekt ") + projName;
       result += " programem";
       return result;
     }
 
     /*! this text is put before a class diagram */
-    virtual QCString trClassDiagram(const char *clName)
+    virtual QCString trClassDiagram(const QCString &clName)
     {
       return QCString("Diagram dědičnosti pro třídu ") + clName;
     }
@@ -619,7 +629,7 @@ class TranslatorCzech : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! used as the title of the HTML page of a class/struct/union */
-    virtual QCString trCompoundReference(const char *clName,
+    virtual QCString trCompoundReference(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -641,7 +651,7 @@ class TranslatorCzech : public Translator
     }
 
     /*! used as the title of the HTML page of a file */
-    virtual QCString trFileReference(const char *fileName)
+    virtual QCString trFileReference(const QCString &fileName)
     {
       QCString result("Dokumentace souboru ");
       result+=fileName;
@@ -649,7 +659,7 @@ class TranslatorCzech : public Translator
     }
 
     /*! used as the title of the HTML page of a namespace */
-    virtual QCString trNamespaceReference(const char *namespaceName)
+    virtual QCString trNamespaceReference(const QCString &namespaceName)
     {
       QCString result("Dokumentace prostoru jmen ");
       result+=namespaceName;
@@ -861,12 +871,12 @@ class TranslatorCzech : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! this text is put before a collaboration diagram */
-    virtual QCString trCollaborationDiagram(const char *clName)
+    virtual QCString trCollaborationDiagram(const QCString &clName)
     {
       return (QCString)"Diagram tříd pro "+clName+":";
     }
     /*! this text is put before an include dependency graph */
-    virtual QCString trInclDepGraph(const char *fName)
+    virtual QCString trInclDepGraph(const QCString &fName)
     {
       return (QCString)"Graf závislostí na vkládaných souborech "
                     "pro "+fName+":";
@@ -942,7 +952,7 @@ class TranslatorCzech : public Translator
     }
     virtual QCString trPublicAttribs()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Datové položky";
       }
@@ -1151,7 +1161,7 @@ class TranslatorCzech : public Translator
       return "Třídy";
     }
     /*! Used as the title of a Java package */
-    virtual QCString trPackage(const char *name)
+    virtual QCString trPackage(const QCString &name)
     {
       return QCString("Balík ") + name;
     }
@@ -1524,7 +1534,7 @@ class TranslatorCzech : public Translator
     /*! This returns the title of a directory page. The name of the
      *  directory is passed via \a dirName.
      */
-    virtual QCString trDirReference(const char *dirName)
+    virtual QCString trDirReference(const QCString &dirName)
     {
         QCString result = "Reference k adresáři ";
         result += dirName;
@@ -1660,7 +1670,7 @@ class TranslatorCzech : public Translator
     }
 
     /*! used as the title of the HTML page of a module/type (Fortran) */
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -1682,7 +1692,7 @@ class TranslatorCzech : public Translator
 
     }
     /*! used as the title of the HTML page of a module (Fortran) */
-    virtual QCString trModuleReference(const char *namespaceName)
+    virtual QCString trModuleReference(const QCString &namespaceName)
     {
       QCString result="Dokumentace modulu ";
       result += namespaceName;
@@ -1782,7 +1792,7 @@ class TranslatorCzech : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! directory relation for \a name */
-    virtual QCString trDirRelation(const char *name)
+    virtual QCString trDirRelation(const QCString &name)
     {
       return "Relace " + QCString(name);
     }
@@ -1819,7 +1829,7 @@ class TranslatorCzech : public Translator
      *  table is shown. The heading for the first column mentions the
      *  source file that has a relation to another file.
      */
-    virtual QCString trFileIn(const char *name)
+    virtual QCString trFileIn(const QCString &name)
     {
       return (QCString)"Soubor v "+name;
     }
@@ -1828,7 +1838,7 @@ class TranslatorCzech : public Translator
      *  table is shown. The heading for the second column mentions the
      *  destination file that is included.
      */
-    virtual QCString trIncludesFileIn(const char *name)
+    virtual QCString trIncludesFileIn(const QCString &name)
     {
       return (QCString)"Vkládá (include) soubor z "+name;
     }
@@ -1873,7 +1883,7 @@ class TranslatorCzech : public Translator
     { return "Copyright"; }
 
     /*! Header for the graph showing the directory dependencies */
-    virtual QCString trDirDepGraph(const char *name)
+    virtual QCString trDirDepGraph(const QCString &name)
     { return QCString("Graf závislosti na adresářích pro  ")+name+":"; }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1903,11 +1913,11 @@ class TranslatorCzech : public Translator
     }
 
     /*! Header of a Java enum page (Java enums are represented as classes). */
-    virtual QCString trEnumReference(const char *name)
+    virtual QCString trEnumReference(const QCString &name)
     { return "Reference k výčtu "+QCString(name); }
 
     /*! Used for a section containing inherited members */
-    virtual QCString trInheritedFrom(const char *members,const char *what)
+    virtual QCString trInheritedFrom(const QCString &members,const QCString &what)
     { return QCString(members)+" dědí se z "+what; }
 
     /*! Header of the sections with inherited members specific for the
@@ -1936,7 +1946,7 @@ class TranslatorCzech : public Translator
      */
     virtual QCString trProvidedByCategory()
     {
-      return "Deklarováno v kategorii @1.";
+      return "Deklarováno v kategorii @0.";
     }
 
     /*! Used in a method of an Objective-C category that extends a class.
@@ -1945,7 +1955,7 @@ class TranslatorCzech : public Translator
      */
     virtual QCString trExtendsClass()
     {
-      return "Rozšiřuje třídu @1.";
+      return "Rozšiřuje třídu @0.";
     }
 
     /*! Used as the header of a list of class methods in Objective-C.
@@ -1971,14 +1981,6 @@ class TranslatorCzech : public Translator
       return "Dokumentace metody";
     }
 
-    /*! Used as the title of the design overview picture created for the
-     *  VHDL output.
-     */
-    virtual QCString trDesignOverview()
-    {
-      return "Návrhové schéma";
-    }
-
 //////////////////////////////////////////////////////////////////////////
 // new since 1.8.4
 //////////////////////////////////////////////////////////////////////////
@@ -1996,21 +1998,21 @@ class TranslatorCzech : public Translator
     { return "Konstantní skupiny"; }
 
     /** UNO IDL constant groups */
-    virtual QCString trConstantGroupReference(const char *namespaceName)
+    virtual QCString trConstantGroupReference(const QCString &namespaceName)
     {
       QCString result="Konstantní skupiny z ";
       result += namespaceName;
       return result;
     }
     /** UNO IDL service page title */
-    virtual QCString trServiceReference(const char *sName)
+    virtual QCString trServiceReference(const QCString &sName)
     {
       QCString result="Popis služby ";
       result += sName;
       return result;
     }
     /** UNO IDL singleton page title */
-    virtual QCString trSingletonReference(const char *sName)
+    virtual QCString trSingletonReference(const QCString &sName)
     {
       QCString result="Popis singletonu ";
       result += sName;
@@ -2035,8 +2037,6 @@ class TranslatorCzech : public Translator
       return result;
     }
 
-//////////////////////////////////////////////////////////////////////////
-
 };
 
 #endif // TRANSLATOR_CZ_H
