@@ -34,7 +34,7 @@
  * Updated to 1.8.4 by Bartomeu Creus Navarro (17-julio-2013)
  */
 
-class TranslatorSpanish : public Translator
+class TranslatorSpanish : public TranslatorAdapter_1_8_15
 {
   public:
 
@@ -68,6 +68,11 @@ class TranslatorSpanish : public Translator
       return "\\usepackage[spanish]{babel}";
     }
 
+    virtual QCString trISOLang()
+    {
+      return "es";
+    }
+
     // --- Language translation methods -------------------
 
     /*! used in the compound documentation before a list of related functions. */
@@ -97,7 +102,7 @@ class TranslatorSpanish : public Translator
     /*! header that is put before the list of member attributes. */
     virtual QCString trMemberDataDocumentation()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Documentación de los campos";
       }
@@ -130,9 +135,9 @@ class TranslatorSpanish : public Translator
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
      */
-    virtual QCString trGeneratedAutomatically(const char *s)
+    virtual QCString trGeneratedAutomatically(const QCString &s)
     { QCString result="Generado automáticamente por Doxygen";
-      if (s) result+=(QCString)" para "+s;
+      if (!s.isEmpty()) result+=(QCString)" para "+s;
       result+=" del código fuente.";
       return result;
     }
@@ -164,7 +169,7 @@ class TranslatorSpanish : public Translator
     /*! This is put above each page as a link to the list of annotated classes */
     virtual QCString trCompoundList()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Estructura de datos";
       }
@@ -181,7 +186,7 @@ class TranslatorSpanish : public Translator
     /*! This is put above each page as a link to all members of compounds. */
     virtual QCString trCompoundMembers()
     {
-    if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+    if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Campos de datos";
       }
@@ -194,7 +199,7 @@ class TranslatorSpanish : public Translator
     /*! This is put above each page as a link to all members of files. */
     virtual QCString trFileMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Globales";
       }
@@ -234,10 +239,14 @@ class TranslatorSpanish : public Translator
     /*! This is an introduction to the annotated compound list. */
     virtual QCString trCompoundListDescription()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Lista de estructuras con una breve descripción:";
       }
+      else if (Config_getBool(OPTIMIZE_OUTPUT_SLICE))
+      {
+        return "Lista de las clases con una breve descripción:";
+      }
       else
       {
        return "Lista de las clases, estructuras, "
@@ -249,7 +258,7 @@ class TranslatorSpanish : public Translator
     virtual QCString trCompoundMembersDescription(bool extractAll)
     {
       QCString result="Lista de todos los ";
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result+="campos de estructuras y uniones";
       }
@@ -264,7 +273,7 @@ class TranslatorSpanish : public Translator
       result+=" con enlaces a ";
       if (!extractAll)
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result+="la documentación de la estructura/unión para cada campo:";
         }
@@ -275,7 +284,7 @@ class TranslatorSpanish : public Translator
       }
       else
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result+="las estructuras/uniones a que pertenecen:";
         }
@@ -291,7 +300,7 @@ class TranslatorSpanish : public Translator
     virtual QCString trFileMembersDescription(bool extractAll)
     {
       QCString result="Lista de ";
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result+="todas las funciones, variables, 'defines', enumeraciones y 'typedefs'";
       }
@@ -343,7 +352,7 @@ class TranslatorSpanish : public Translator
      */
     virtual QCString trCompoundIndex()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Índice de estructura de datos";
       }
@@ -370,7 +379,7 @@ class TranslatorSpanish : public Translator
      */
     virtual QCString trClassDocumentation()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Documentación de las estructuras de datos";
       }
@@ -406,13 +415,13 @@ class TranslatorSpanish : public Translator
      *  list of defines
      */
     virtual QCString trDefines()
-    { return "'defines'"; }
+    { return "defines"; }
 
     /*! This is used in the documentation of a file as a header before the
      *  list of typedefs
      */
     virtual QCString trTypedefs()
-    { return "'typedefs'"; }
+    { return "typedefs"; }
 
     /*! This is used in the documentation of a file as a header before the
      *  list of enumerations
@@ -473,7 +482,7 @@ class TranslatorSpanish : public Translator
      */
     virtual QCString trCompounds()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Estructuras de datos";
       }
@@ -486,16 +495,16 @@ class TranslatorSpanish : public Translator
     /*! This is used in the standard footer of each page and indicates when
      *  the page was generated
      */
-    virtual QCString trGeneratedAt(const char *date,const char *projName)
+    virtual QCString trGeneratedAt(const QCString &date,const QCString &projName)
     {
       QCString result=(QCString)"Generado el "+date;
-      if (projName) result+=(QCString)" para "+projName;
+      if (!projName.isEmpty()) result+=(QCString)" para "+projName;
       result+=(QCString)" por";
       return result;
     }
 
     /*! this text is put before a class diagram */
-    virtual QCString trClassDiagram(const char *clName)
+    virtual QCString trClassDiagram(const QCString &clName)
     {
       return (QCString)"Diagrama de herencias de "+clName;
     }
@@ -574,7 +583,7 @@ class TranslatorSpanish : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! used as the title of the HTML page of a class/struct/union */
-    virtual QCString trCompoundReference(const char *clName,
+    virtual QCString trCompoundReference(const QCString &clName,
                                          ClassDef::CompoundType compType,
                                          bool isTemplate)
     {
@@ -596,7 +605,7 @@ class TranslatorSpanish : public Translator
     }
 
     /*! used as the title of the HTML page of a file */
-    virtual QCString trFileReference(const char *fileName)
+    virtual QCString trFileReference(const QCString &fileName)
     {
       QCString result="Referencia del Archivo ";
       result+=fileName;
@@ -604,7 +613,7 @@ class TranslatorSpanish : public Translator
     }
 
     /*! used as the title of the HTML page of a namespace */
-    virtual QCString trNamespaceReference(const char *namespaceName)
+    virtual QCString trNamespaceReference(const QCString &namespaceName)
     {
       QCString result="Referencia del Namespace ";
       result+=namespaceName;
@@ -811,13 +820,13 @@ class TranslatorSpanish : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! this text is put before a collaboration diagram */
-    virtual QCString trCollaborationDiagram(const char *clName)
+    virtual QCString trCollaborationDiagram(const QCString &clName)
     {
       return (QCString)"Diagrama de colaboración para "+clName+":";
     }
 
     /*! this text is put before an include dependency graph */
-    virtual QCString trInclDepGraph(const char *fName)
+    virtual QCString trInclDepGraph(const QCString &fName)
     {
       return (QCString)"Dependencia gráfica adjunta para "+fName+":";
     }
@@ -906,7 +915,7 @@ class TranslatorSpanish : public Translator
 
     virtual QCString trPublicAttribs()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Campos de datos";
       }
@@ -1119,7 +1128,7 @@ class TranslatorSpanish : public Translator
     /*! Used for Java classes in the summary section of Java packages */
     virtual QCString trClasses()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Estructuras de Datos";
       }
@@ -1130,7 +1139,7 @@ class TranslatorSpanish : public Translator
     }
 
     /*! Used as the title of a Java package */
-    virtual QCString trPackage(const char *name)
+    virtual QCString trPackage(const QCString &name)
     {
       return (QCString)"Paquetes "+name;
     }
@@ -1537,7 +1546,7 @@ class TranslatorSpanish : public Translator
     /*! This returns the title of a directory page. The name of the
      *  directory is passed via \a dirName.
      */
-    virtual QCString trDirReference(const char *dirName)
+    virtual QCString trDirReference(const QCString &dirName)
     {
       QCString result="Referencia del directorio ";
       result+=dirName;
@@ -1675,7 +1684,7 @@ class TranslatorSpanish : public Translator
     }
 
     /*! used as the title of the HTML page of a module/type (Fortran) */
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -1697,7 +1706,7 @@ class TranslatorSpanish : public Translator
     }
 
     /*! used as the title of the HTML page of a module (Fortran) */
-    virtual QCString trModuleReference(const char *namespaceName)
+    virtual QCString trModuleReference(const QCString &namespaceName)
     {
       QCString result="Referencia módulo ";
       result+=namespaceName;
@@ -1800,7 +1809,7 @@ class TranslatorSpanish : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! directory relation for \a name */
-    virtual QCString trDirRelation(const char *name)
+    virtual QCString trDirRelation(const QCString &name)
     {
       return QCString(name)+" relación";
     }
@@ -1837,7 +1846,7 @@ class TranslatorSpanish : public Translator
      *  table is shown. The heading for the first column mentions the
      *  source file that has a relation to another file.
      */
-    virtual QCString trFileIn(const char *name)
+    virtual QCString trFileIn(const QCString &name)
     {
       return (QCString)"Fichero en "+name;
     }
@@ -1846,7 +1855,7 @@ class TranslatorSpanish : public Translator
      *  table is shown. The heading for the second column mentions the
      *  destination file that is included.
      */
-    virtual QCString trIncludesFileIn(const char *name)
+    virtual QCString trIncludesFileIn(const QCString &name)
     {
       return (QCString)"Incluye ficheros en "+name;
     }
@@ -1894,7 +1903,7 @@ class TranslatorSpanish : public Translator
     { return "Copyright"; }
 
     /*! Header for the graph showing the directory dependencies */
-    virtual QCString trDirDepGraph(const char *name)
+    virtual QCString trDirDepGraph(const QCString &name)
     { return QCString("Gráfico de dependencias de directorios para ")+name+":"; }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1924,11 +1933,11 @@ class TranslatorSpanish : public Translator
     }
 
     /*! Header of a Java enum page (Java enums are represented as classes). */
-    virtual QCString trEnumReference(const char *name)
+    virtual QCString trEnumReference(const QCString &name)
     { return QCString("Referencia del enum ") + QCString(name); }
 
     /*! Used for a section containing inherited members */
-    virtual QCString trInheritedFrom(const char *members,const char *what)
+    virtual QCString trInheritedFrom(const QCString &members,const QCString &what)
     { return QCString(members)+" heredados desde "+what; }
 
     /*! Header of the sections with inherited members specific for the
@@ -1957,7 +1966,7 @@ class TranslatorSpanish : public Translator
      */
     virtual QCString trProvidedByCategory()
     {
-      return "Proporcionado por categoría @1.";
+      return "Proporcionado por categoría @0.";
     }
 
     /*! Used in a method of an Objective-C category that extends a class.
@@ -1966,7 +1975,7 @@ class TranslatorSpanish : public Translator
      */
     virtual QCString trExtendsClass()
     {
-      return "Extiende la clase @1.";
+      return "Extiende la clase @0.";
     }
 
     /*! Used as the header of a list of class methods in Objective-C.
@@ -1992,14 +2001,6 @@ class TranslatorSpanish : public Translator
       return "Método de documentación";
     }
 
-    /*! Used as the title of the design overview picture created for the
-     *  VHDL output.
-     */
-    virtual QCString trDesignOverview()
-    {
-      return "Diseño información general";
-    }
-
 //////////////////////////////////////////////////////////////////////////
 // new since 1.8.4
 //////////////////////////////////////////////////////////////////////////
@@ -2017,21 +2018,21 @@ class TranslatorSpanish : public Translator
     { return "Grupos constantes"; }
 
     /** UNO IDL constant groups */
-    virtual QCString trConstantGroupReference(const char *namespaceName)
+    virtual QCString trConstantGroupReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+=" referencia de grupos constantes";
       return result;
     }
     /** UNO IDL service page title */
-    virtual QCString trServiceReference(const char *sName)
+    virtual QCString trServiceReference(const QCString &sName)
     {
       QCString result=(QCString)sName;
       result+=" Referencia servicio";
       return result;
     }
     /** UNO IDL singleton page title */
-    virtual QCString trSingletonReference(const char *sName)
+    virtual QCString trSingletonReference(const QCString &sName)
     {
       QCString result=(QCString)sName;
       result+=" referencia Singleton";
@@ -2057,6 +2058,252 @@ class TranslatorSpanish : public Translator
     }
 
 //////////////////////////////////////////////////////////////////////////
+// new since 1.8.15
+//////////////////////////////////////////////////////////////////////////
+
+    /** VHDL design unit hierarchy */
+    virtual QCString trDesignUnitHierarchy()
+    { return "Jerarquía de unidades de diseño"; }
+    /** VHDL design unit list */
+    virtual QCString trDesignUnitList()
+    { return "Lista de unidades de diseño"; }
+    /** VHDL design unit members */
+    virtual QCString trDesignUnitMembers()
+    { return "Miembros de las unidades de diseño"; }
+    /** VHDL design unit list description */
+    virtual QCString trDesignUnitListDescription()
+    {
+        return "Aquí hay una lista de todos los miembros de la unidad de diseño con "
+            "enlaces a las entidades a las que pertenecen:";
+    }
+    /** VHDL design unit index */
+    virtual QCString trDesignUnitIndex()
+    { return "Indice de unidades de diseño"; }
+    /** VHDL design units */
+    virtual QCString trDesignUnits()
+    { return "Unidades de Diseño"; }
+    /** VHDL functions/procedures/processes */
+    virtual QCString trFunctionAndProc()
+    { return "Funciones/Procedimientos/Procesos"; }
+    /** VHDL type */
+    virtual QCString trVhdlType(uint64 type,bool single)
+    {
+      switch(type)
+      {
+        case VhdlDocGen::LIBRARY:
+          if (single) return "Libreria";
+          else        return "Librerias";
+        case VhdlDocGen::PACKAGE:
+          if (single) return "Paquete";
+          else        return "Paquetes";
+        case VhdlDocGen::SIGNAL:
+          if (single) return "Señal";
+          else        return "Señales";
+        case VhdlDocGen::COMPONENT:
+          if (single) return "Componente";
+          else        return "Componentes";
+        case VhdlDocGen::CONSTANT:
+          if (single) return "Constante";
+          else        return "Constantes";
+        case VhdlDocGen::ENTITY:
+          if (single) return "Entidad";
+          else        return "Entidades";
+        case VhdlDocGen::TYPE:
+          if (single) return "Tipo";
+          else        return "Tipos";
+        case VhdlDocGen::SUBTYPE:
+          if (single) return "Subtipo";
+          else        return "Subtipos";
+        case VhdlDocGen::FUNCTION:
+          if (single) return "Función";
+          else        return "Funciones";
+        case VhdlDocGen::RECORD:
+          if (single) return "Registro";
+          else        return "Registros";
+        case VhdlDocGen::PROCEDURE:
+          if (single) return "Procedimiento";
+          else        return "Procedimientos";
+        case VhdlDocGen::ARCHITECTURE:
+          if (single) return "Arquitectura";
+          else        return "Arquitecturas";
+        case VhdlDocGen::ATTRIBUTE:
+          if (single) return "Atributo";
+          else        return "Atributos";
+        case VhdlDocGen::PROCESS:
+          if (single) return "Proceso";
+          else        return "Procesos";
+        case VhdlDocGen::PORT:
+          if (single) return "Puerto";
+          else        return "Puertos";
+        case VhdlDocGen::USE:
+          if (single) return "cláusula de uso";
+          else        return "Cláusulas de uso";
+        case VhdlDocGen::GENERIC:
+          if (single) return "Genérico";
+          else        return "Genéricos";
+        case VhdlDocGen::PACKAGE_BODY:
+          return "Cuerpo del paquete";
+        case VhdlDocGen::UNITS:
+          return "Unidades";
+        case VhdlDocGen::SHAREDVARIABLE:
+          if (single) return "Variable Compartida";
+          else        return "Variable Compartidas";
+        case VhdlDocGen::VFILE:
+          if (single) return "Fichero";
+          else        return "Ficheros";
+        case VhdlDocGen::GROUP:
+          if (single) return "Grupo";
+          else        return "Grupos";
+        case VhdlDocGen::INSTANTIATION:
+          if (single) return "Instanciación";
+          else        return "Instanciaciones";
+        case VhdlDocGen::ALIAS:
+          if (single) return "Alias";
+          else        return "Aliases";
+        case VhdlDocGen::CONFIG:
+          if (single) return "Configuración";
+          else        return "Configuraciones";
+        case VhdlDocGen::MISCELLANEOUS:
+          return "Varios";
+        case VhdlDocGen::UCF_CONST:
+          return "Restricciones";
+        default:
+          return "Clase";
+      }
+    }
+    virtual QCString trCustomReference(const QCString &name)
+    { return "Referencia"+QCString(name); }
+
+    virtual QCString trConstants()
+    {
+        return "Constantes";
+    }
+    virtual QCString trConstantDocumentation()
+    {
+        return "Documentación de constante";
+    }
+    virtual QCString trSequences()
+    {
+        return "Secuencias";
+    }
+    virtual QCString trSequenceDocumentation()
+    {
+        return "Documentación de secuencia";
+    }
+    virtual QCString trDictionaries()
+    {
+        return "Diccionarios";
+    }
+    virtual QCString trDictionaryDocumentation()
+    {
+        return "Documentación de diccionario";
+    }
+    virtual QCString trSliceInterfaces()
+    {
+        return "Interfaces";
+    }
+    virtual QCString trInterfaceIndex()
+    {
+        return "Indice de interfaces";
+    }
+    virtual QCString trInterfaceList()
+    {
+        return "Lista de interfaces";
+    }
+    virtual QCString trInterfaceListDescription()
+    {
+        return "Aquí están las interfaces con una breve descripción:";
+    }
+    virtual QCString trInterfaceHierarchy()
+    {
+        return "Jerarquía de interfaces";
+    }
+    virtual QCString trInterfaceHierarchyDescription()
+    {
+        return "Este listado de herencia está ordenado de forma aproximada, pero no completamente en orden alfabético:";
+    }
+    virtual QCString trInterfaceDocumentation()
+    {
+        return "Documentación de interfaz";
+    }
+    virtual QCString trStructs()
+    {
+        return "Estructuras";
+    }
+    virtual QCString trStructIndex()
+    {
+        return "Indice de estructuras";
+    }
+    virtual QCString trStructList()
+    {
+        return "Lista de estructuras";
+    }
+    virtual QCString trStructListDescription()
+    {
+        return "Aquí están las estructuras de datos con una breve descripción:";
+    }
+    virtual QCString trStructDocumentation()
+    {
+        return "Documentación estructura";
+    }
+    virtual QCString trExceptionIndex()
+    {
+        return "Indice de excepciones";
+    }
+    virtual QCString trExceptionList()
+    {
+        return "Lista de excepciones";
+    }
+    virtual QCString trExceptionListDescription()
+    {
+        return "Aquí están las excepciones con una breve descripción:";
+    }
+    virtual QCString trExceptionHierarchy()
+    {
+        return "Jerarquía de excepciones";
+    }
+    virtual QCString trExceptionHierarchyDescription()
+    {
+        return "Este listado de herencia está ordenado de forma aproximada, pero no completamente en orden alfabético:";
+    }
+    virtual QCString trExceptionDocumentation()
+    {
+        return "Documentación de excepción";
+    }
+    virtual QCString trCompoundReferenceSlice(const QCString &clName, ClassDef::CompoundType compType, bool isLocal)
+    {
+      QCString result="Referencia de";
+      switch(compType)
+      {
+        case ClassDef::Class:      result+=" la Clase"; break;
+        case ClassDef::Struct:     result+=" la Estructura"; break;
+        case ClassDef::Union:      result+=" la Unión"; break;
+        case ClassDef::Interface:  result+=" la Interfaz"; break;
+        case ClassDef::Protocol:   result+="l Protocolo"; break;
+        case ClassDef::Category:   result+=" la Categoría"; break;
+        case ClassDef::Exception:  result+=" la Excepción"; break;
+        default: break;
+      }
+      if (isLocal) result+=" Local";
+      result+=(QCString)clName;
+      return result;
+    }
+    virtual QCString trOperations()
+    {
+        return "Operaciones";
+    }
+    virtual QCString trOperationDocumentation()
+    {
+        return "Documentación de operación";
+    }
+    virtual QCString trDataMembers()
+    {
+        return "Miembros de datos";
+    }
+    virtual QCString trDataMemberDocumentation()
+    {
+        return "Documentación miembro de datos";
+    }
 
 };
 
