@@ -2,7 +2,7 @@
  *
  *
  *
- * Copyright (C) 1997-2014 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -92,6 +92,8 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     { return "croatian"; }
     QCString latexLanguageSupportCommand()
     { return "\\usepackage[croatian]{babel}\n"; }
+    QCString trISOLang()
+    { return "hr"; }
     QCString trRelatedFunctions()
     { return "Povezane funkcije"; }
     QCString trRelatedSubscript()
@@ -116,9 +118,9 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     { return "Ovo je popis svih članova"; }
     QCString trIncludingInheritedMembers()
     { return ", uključujući naslijeđene članove."; }
-    QCString trGeneratedAutomatically(const char *s)
+    QCString trGeneratedAutomatically(const QCString &s)
     { QCString result="napravljeno automatski Doxygen-om";
-      if (s) result+=(QCString)" za "+s;
+      if (!s.isEmpty()) result+=(QCString)" za "+s;
       result+=" iz programskog koda.";
       return result;
     }
@@ -134,7 +136,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     { return "Stablo klasa"; }
     QCString trCompoundList()
 	{
-		if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+		if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
 		{
 			return "Sve strukture";
 		}
@@ -147,7 +149,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     { return "Popis datoteka"; }
     QCString trCompoundMembers()
 	{
-		if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+		if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
 			return "Svi članovi struktura";
 		else
 			return "Svi članovi klasa";
@@ -181,7 +183,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
       if (!extractAll)
 		  result+="dokumentiranih ";
 
-	  if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+	  if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
 		  result+="članova klasa s linkovima na ";
 	  else
 		  result+="članova struktura s linkovima na ";
@@ -192,7 +194,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
       }
 	  else
 	  {
-		  if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+		  if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
 			  result+="dokumentaciju klase :";
 		  else
 			  result +="dokumentaciju strukture";
@@ -226,7 +228,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     { return "Hijerarhijsko kazalo"; }
     QCString trCompoundIndex()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Kazalo struktura podataka";
       }
@@ -241,7 +243,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     { return "Dokumentacija modula"; }
     QCString trClassDocumentation()
     {
-		if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+		if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
 		{
 			return "Dokumentacija struktura podataka";
 		}
@@ -283,14 +285,14 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     { return "Dokumentacija varijable"; }
     QCString trCompounds()
     { return "Strukture"; }
-    QCString trGeneratedAt(const char *date,const char *projName)
+    QCString trGeneratedAt(const QCString &date,const QCString &projName)
     {
       QCString result=(QCString)"Napravljeno "+date;
-      if (projName) result+=(QCString)" projekt: "+projName;
+      if (!projName.isEmpty()) result+=(QCString)" projekt: "+projName;
       result+=" generator: ";
       return result;
     }
-    QCString trClassDiagram(const char *clName)
+    QCString trClassDiagram(const QCString &clName)
     {
       return QCString("Dijagram klasa za ")+clName;
     }
@@ -340,7 +342,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
 // new since 0.49-990425
 //////////////////////////////////////////////////////////////////////////
 
-    QCString trCompoundReference(const char *clName,
+    QCString trCompoundReference(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool /*isTemplate*/)
       // used as the title of the HTML page of a class/struct/union
@@ -360,14 +362,14 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
       result += clName;
       return result;
     }
-    QCString trFileReference(const char *fileName)
+    QCString trFileReference(const QCString &fileName)
       // used as the title of the HTML page of a file
     {
       QCString result="Opis datoteke ";
           result+=fileName;
           return result;
     }
-    QCString trNamespaceReference(const char *namespaceName)
+    QCString trNamespaceReference(const QCString &namespaceName)
       // used as the title of the HTML page of a namespace
     {
       QCString result ="Opis imenika ";
@@ -557,12 +559,12 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
 //////////////////////////////////////////////////////////////////////////
 
     /*! this text is put before a collaboration diagram */
-    QCString trCollaborationDiagram(const char *clName)
+    QCString trCollaborationDiagram(const QCString &clName)
     {
       return (QCString)"Kolaboracijski dijagram za "+clName+ ":";
     }
     /*! this text is put before an include dependency graph */
-    QCString trInclDepGraph(const char *fName)
+    QCString trInclDepGraph(const QCString &fName)
     {
       return (QCString)"Graf include međuovisnosti za "+fName+":";
     }
@@ -753,7 +755,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
         "};\n"
         "\\endcode\n"
         "To će rezultirati grafikonom:"
-        "<p><center><img src=\"graph_legend."+Config_getEnum("DOT_IMAGE_FORMAT")+"\"></center>\n"
+        "<p><center><img src=\"graph_legend."+getDotImageExtension()+"\"></center>\n"
         "<p>\n"
         "Pravokutnici imaju slijedeće značenje:\n"
         "<ul>\n"
@@ -822,7 +824,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
       return "Klase";
     }
     /*! Used as the title of a Java package */
-    virtual QCString trPackage(const char *name)
+    virtual QCString trPackage(const QCString &name)
     {
       return (QCString)"Paket "+name;
     }
@@ -1192,7 +1194,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     /*! This returns the title of a directory page. The name of the
      *  directory is passed via \a dirName.
      */
-    virtual QCString trDirReference(const char *dirName)
+    virtual QCString trDirReference(const QCString &dirName)
     { QCString result= "Opis direktorija "; result += dirName; return result; }
 
     /*! This returns the word directory with or without starting capital
@@ -1318,7 +1320,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     }
 
     /*! used as the title of the HTML page of a module/type (Fortran) */
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
       ClassDef::CompoundType compType,
       bool isTemplate)
     {
@@ -1339,7 +1341,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
       return result;
     }
     /*! used as the title of the HTML page of a module (Fortran) */
-    virtual QCString trModuleReference(const char *namespaceName)
+    virtual QCString trModuleReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+=" - Sadržaj modula";
@@ -1436,7 +1438,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     //////////////////////////////////////////////////////////////////////////
 
     /*! directory relation for \a name */
-    virtual QCString trDirRelation(const char *name)
+    virtual QCString trDirRelation(const QCString &name)
     {
         return QCString("Relacije ") + QCString(name);
     }
@@ -1473,7 +1475,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
      *  table is shown. The heading for the first column mentions the
      *  source file that has a relation to another file.
      */
-    virtual QCString trFileIn(const char *name)
+    virtual QCString trFileIn(const QCString &name)
     {
       return (QCString)"Datoteka u "+name;
     }
@@ -1482,7 +1484,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
      *  table is shown. The heading for the second column mentions the
      *  destination file that is included.
      */
-    virtual QCString trIncludesFileIn(const char *name)
+    virtual QCString trIncludesFileIn(const QCString &name)
     {
       return (QCString)"Uključuje datotake u "+name;
     }
@@ -1526,7 +1528,7 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     { return "Copyright"; }
 
     /*! Header for the graph showing the directory dependencies */
-    virtual QCString trDirDepGraph(const char *name)
+    virtual QCString trDirDepGraph(const QCString &name)
     { return QCString("Direktoriji o kojima ovisi ")+name+":"; }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1554,11 +1556,11 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     }
 
     /*! Header of a Java enum page (Java enums are represented as classes). */
-    virtual QCString trEnumReference(const char *name)
+    virtual QCString trEnumReference(const QCString &name)
     { return "Opis enumeracije " + QCString(name); }
 
     /*! Used for a section containing inherited members */
-    virtual QCString trInheritedFrom(const char *members,const char *what)
+    virtual QCString trInheritedFrom(const QCString &members,const QCString &what)
     { return QCString(members)+" naslijeđeni od "+what; }
 
     /*! Header of the sections with inherited members specific for the
@@ -1567,8 +1569,6 @@ class TranslatorCroatian : public TranslatorAdapter_1_8_2
     virtual QCString trAdditionalInheritedMembers()
     { return "Dodatni naslijeđeni članovi"; }
 
-//////////////////////////////////////////////////////////////////////////
-
 };
 
 #endif
